@@ -113,6 +113,10 @@ class QueueBuffer(_BufferAbstract):
         if buffer:
             self._data_queue.append(buffer)
 
+    def qsize(self):
+
+        return self._data_queue.size()
+
 
 class FileBuffer(ContextManager):
     """文件缓存类
