@@ -12,7 +12,7 @@ from datetime import timedelta
 
 from loguru._file_sink import FileSink
 
-from elasticsearch import Elasticsearch, helpers as es_helpers
+from elasticsearch import Elasticsearch, NotFoundError as ESNotFoundError, helpers as es_helpers
 
 from .trace import get_trace_id
 from .base import Utils
@@ -73,142 +73,149 @@ class InterceptHandler(logging.Handler):
 class ElasticsearchDataStreamUtil:
 
     def __init__(
-            self, elasticsearch: Elasticsearch, alias_name: str, *,
+            self, elasticsearch: Elasticsearch, stream_name: str, *,
             rollover_max_age: str = r'1d', rollover_max_primary_shard_size: str = r'50gb',
-            delete_min_age: str = r'30d', refresh_interval: str = r'5s', timestamp_order: str = r'desc'
+            delete_min_age: str = r'30d', refresh_interval: str = r'5s', number_of_replicas=0,
+            timestamp_order: str = r'desc'
     ):
 
         self._elasticsearch = elasticsearch
 
-        self._alias_name = alias_name
-        self._policy_name = f'{alias_name}-ilm-policy'
+        self._stream_name = stream_name
+        self._policy_name = f'{stream_name}-ilm-policy'
 
         self._rollover_max_age = rollover_max_age
         self._rollover_max_primary_shard_size = rollover_max_primary_shard_size
 
         self._delete_min_age = delete_min_age
         self._refresh_interval = refresh_interval
+        self._number_of_replicas = number_of_replicas
         self._timestamp_order = timestamp_order
 
     def initialize(self):
 
         try:
-
-            indices = self._elasticsearch.indices
-
-            if not indices.exists_alias(name=self._alias_name):
-                self._create_lifecycle()
-                self._create_index_template()
-
-        except Exception as err:
-
-            sys.stderr.write(str(err))
+            self._elasticsearch.indices.get_data_stream(name=self._stream_name)
+        except ESNotFoundError as _:
+            self._create_lifecycle()
+            self._create_index_template()
 
     def _create_lifecycle(self):
 
-        policy = {
-            r'phases': {
-                r'hot': {
-                    r'actions': {
-                        r'rollover': {
-                            r'max_age': self._rollover_max_age,
-                            r'max_primary_shard_size': self._rollover_max_primary_shard_size,
-                        },
-                        r'set_priority': {
-                            r'priority': 100,
-                        }
-                    },
-                    r'min_age': r'0ms',
-                },
-                r'delete': {
-                    r'actions': {
-                        r'delete': {}
-                    },
-                    r'min_age': self._delete_min_age,
-                },
-            },
-        }
+        try:
 
-        self._elasticsearch.ilm.put_lifecycle(name=self._policy_name, policy=policy)
+            policy = {
+                r'phases': {
+                    r'hot': {
+                        r'actions': {
+                            r'rollover': {
+                                r'max_age': self._rollover_max_age,
+                                r'max_primary_shard_size': self._rollover_max_primary_shard_size,
+                            },
+                            r'set_priority': {
+                                r'priority': 100,
+                            }
+                        },
+                        r'min_age': r'0ms',
+                    },
+                    r'delete': {
+                        r'actions': {
+                            r'delete': {}
+                        },
+                        r'min_age': self._delete_min_age,
+                    },
+                },
+            }
+
+            self._elasticsearch.ilm.put_lifecycle(name=self._policy_name, policy=policy)
+
+        except Exception as err:
+
+            sys.stderr.write(f'{str(err.body)}\n')
 
     def _create_index_template(self):
 
-        mappings = {
-            r'dynamic': r'strict',
-            r'properties': {
-                r'extra': {
-                    r'type': r'flattened',
-                },
-                r'process': {
-                    r'properties': {
-                        r'id': {
-                            r'type': r'keyword',
-                        },
-                        r'name': {
-                            r'type': r'keyword',
-                            r'ignore_above': 64,
+        try:
+
+            mappings = {
+                r'dynamic': r'strict',
+                r'properties': {
+                    r'extra': {
+                        r'type': r'flattened',
+                    },
+                    r'process': {
+                        r'properties': {
+                            r'id': {
+                                r'type': r'keyword',
+                            },
+                            r'name': {
+                                r'type': r'keyword',
+                                r'ignore_above': 64,
+                            },
                         },
                     },
-                },
-                r'thread': {
-                    r'properties': {
-                        r'id': {
-                            r'type': r'keyword',
-                        },
-                        r'name': {
-                            r'type': r'keyword',
-                            r'ignore_above': 64,
+                    r'thread': {
+                        r'properties': {
+                            r'id': {
+                                r'type': r'keyword',
+                            },
+                            r'name': {
+                                r'type': r'keyword',
+                                r'ignore_above': 64,
+                            },
                         },
                     },
-                },
-                r'level': {
-                    r'properties': {
-                        r'no': {
-                            r'type': r'integer',
-                        },
-                        r'name': {
-                            r'type': r'keyword',
-                            r'ignore_above': 64,
+                    r'level': {
+                        r'properties': {
+                            r'no': {
+                                r'type': r'integer',
+                            },
+                            r'name': {
+                                r'type': r'keyword',
+                                r'ignore_above': 64,
+                            },
                         },
                     },
-                },
-                r'module': {
-                    r'type': r'text',
-                },
-                r'message': {
-                    r'type': r'text',
-                    r'analyzer': r'ik_max_word',
-                    r'search_analyzer': r'ik_smart',
-                },
-                r'@timestamp': {
-                    r'type': r'date',
-                    r'format': r'epoch_millis',
-                },
+                    r'module': {
+                        r'type': r'text',
+                    },
+                    r'message': {
+                        r'type': r'text',
+                    },
+                    r'@timestamp': {
+                        r'type': r'date',
+                        r'format': r'epoch_millis',
+                    },
+                }
             }
-        }
 
-        template = {
-            r'settings': {
-                r'index': {
-                    r'lifecycle': {
-                        r'name': self._policy_name,
+            template = {
+                r'settings': {
+                    r'index': {
+                        r'lifecycle': {
+                            r'name': self._policy_name,
+                        },
+                        r'refresh_interval': self._refresh_interval,
+                        r'number_of_replicas': self._number_of_replicas,
+                        r'sort': {
+                            r'field': r'@timestamp',
+                            r'order': self._timestamp_order,
+                        }
                     },
-                    r'refresh_interval': self._refresh_interval,
-                    r'sort': {
-                        r'field': r'@timestamp',
-                        r'order': self._timestamp_order,
-                    }
                 },
-            },
-            r'mappings': mappings,
-        }
+                r'mappings': mappings,
+            }
 
-        self._elasticsearch.indices.put_index_template(
-            name=self._alias_name,
-            template=template,
-            index_patterns=[f'{self._alias_name}*'],
-            data_stream={},
-        )
+            self._elasticsearch.indices.put_index_template(
+                name=self._stream_name,
+                template=template,
+                index_patterns=[f'{self._stream_name}*'],
+                data_stream={},
+            )
+
+        except Exception as err:
+
+            sys.stderr.write(f'{str(err.body)}\n')
 
 
 class ElasticsearchSink:
@@ -224,6 +231,12 @@ class ElasticsearchSink:
 
         self._elasticsearch = Elasticsearch(hosts, **kwargs)
 
+        ElasticsearchDataStreamUtil(
+            self._elasticsearch, index,
+            rollover_max_age=rollover_max_age, rollover_max_primary_shard_size=rollover_max_primary_shard_size,
+            delete_min_age=delete_min_age
+        ).initialize()
+
         self._index = index
 
         self._buffer = queue.Queue(buffer_maxsize)
@@ -232,21 +245,18 @@ class ElasticsearchSink:
         self._task = Thread(target=self._do_task)
         self._task.start()
 
-        ElasticsearchDataStreamUtil(
-            self._elasticsearch, index,
-            rollover_max_age=rollover_max_age, rollover_max_primary_shard_size=rollover_max_primary_shard_size,
-            delete_min_age=delete_min_age
-        ).initialize()
-
     def write(self, message):
 
+        if message.record[r'thread'].id == self._task.ident:
+            return
+
         log_extra = message.record[r'extra']
 
         if r'trace_id' not in log_extra and (trace_id := get_trace_id()) is not None:
             log_extra[r'trace_id'] = trace_id
 
         try:
-            self._buffer.put(message, block=True, timeout=1)
+            self._buffer.put_nowait(message)
         except queue.Full as _:
             if message.record[r'level'].no > logging.INFO:
                 sys.stderr.write(str(message))
@@ -258,42 +268,42 @@ class ElasticsearchSink:
 
     def _do_task(self):
 
-        current_ident = self._task.ident
-
         while not self._closed:
 
+            messages = []
+
             try:
 
-                records = [self._buffer.get(block=True, timeout=1).record]
+                messages.append(self._buffer.get(block=True, timeout=1))
 
                 for _ in range(min(self._buffer.qsize(), 1000)):
-                    records.append(self._buffer.get_nowait().record)
+                    messages.append(self._buffer.get_nowait())
 
-                if records:
+                if messages:
                     es_helpers.bulk(
                         self._elasticsearch,
                         actions=[
                             {
                                 r'_op_type': r'create',
                                 r'_index': self._index,
-                                r'extra': _row[r'extra'],
+                                r'extra': _msg.record[r'extra'],
                                 r'process': {
-                                    r'id': _row[r'process'].id,
-                                    r'name': _row[r'process'].name,
+                                    r'id': _msg.record[r'process'].id,
+                                    r'name': _msg.record[r'process'].name,
                                 },
                                 r'thread': {
-                                    r'id': _row[r'thread'].id,
-                                    r'name': _row[r'thread'].name,
+                                    r'id': _msg.record[r'thread'].id,
+                                    r'name': _msg.record[r'thread'].name,
                                 },
                                 r'level': {
-                                    r'no': _row[r'level'].no,
-                                    r'name': _row[r'level'].name,
+                                    r'no': _msg.record[r'level'].no,
+                                    r'name': _msg.record[r'level'].name,
                                 },
-                                r'module': f"{_row[r'name']}:{_row[r'function']}:{_row[r'line']}",
-                                r'message': _row[r'message'],
-                                r'@timestamp': int(_row[r'time'].timestamp() * 1000),
+                                r'module': f"{_msg.record[r'name']}:{_msg.record[r'function']}:{_msg.record[r'line']}",
+                                r'message': _msg.record[r'message'],
+                                r'@timestamp': int(_msg.record[r'time'].timestamp() * 1000),
                             }
-                            for _row in records if current_ident != _row[r'thread'].id
+                            for _msg in messages
                         ]
                     )
 
@@ -301,9 +311,11 @@ class ElasticsearchSink:
 
                 pass
 
-            except Exception as err:
+            except Exception as _:
 
-                sys.stderr.write(str(err))
+                for _msg in messages:
+                    if _msg.record[r'level'].no > logging.INFO:
+                        sys.stderr.write(str(_msg))
 
 
 class QueuedFileSink(FileSink):
@@ -324,7 +336,7 @@ class QueuedFileSink(FileSink):
     def write(self, message):
 
         try:
-            self._buffer.put(message, block=True, timeout=1)
+            self._buffer.put_nowait(message)
         except queue.Full as _:
             if message.record[r'level'].no > logging.INFO:
                 sys.stderr.write(str(message))
@@ -342,7 +354,7 @@ class QueuedFileSink(FileSink):
 
             try:
                 super().write(
-                    self._buffer.get(block=True, timeout=10)
+                    self._buffer.get(block=True, timeout=1)
                 )
             except queue.Empty as _:
                 pass
