@@ -67,7 +67,7 @@ def tracing(func):
         try:
             return func(*args, **kwargs)
         except Exception as err:
-            raise err
+            logger.error(err)
         finally:
             logger.bind(
                 trace_id=get_trace_id(), trace_no=inc_trace_no(),
@@ -90,7 +90,7 @@ def async_tracing(func):
         try:
             return await func(*args, **kwargs)
         except Exception as err:
-            raise err
+            logger.error(err)
         finally:
             logger.bind(
                 trace_id=get_trace_id(), trace_no=inc_trace_no(),
